/************************************************************************************************

 Author        : ADI - Apps            www.analog.com

 Date          : July 2009

 File          : I2C_Slave.c

 Hardware      : ADuC7122

 Description   : This I2C Slave example shows how to initialise the I2C peripheral in Slave mode.
 				This simple example Stores up to 6 bytes when written to by a Master.
				It will return 6 bytes when a Read request comes in from a Master.		
*************************************************************************************************/
// Bit Definitions
#define BIT0  0x01
#define BIT1  0x02
#define BIT2  0x04
#define BIT3  0x08
#define BIT4  0x10
#define BIT5  0x20
#define BIT6  0x40
#define BIT7  0x80
#define BIT8  0x100
#define BIT9  0x200
#define BIT10 0x400
#define BIT11 0x800
#define BIT12 0x1000
#define BIT13 0x2000
#define BIT14 0x4000
#define BIT15 0x8000
#define BIT16 0x10000
#define BIT20 0x100000

#include <ADuC7122.h>

unsigned char szStoreData[] = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55};	// Array to send to Slave
unsigned char ucStoreCount = 0;  									// Array index variable for szStoreData[]

int main(void)
{
    POWKEY1 = 0x1;
	POWCON = 0x02;		   			// Set core to max CPU speed of 10.24Mhz
	POWKEY2 = 0xF4;

    // Configure P0.0 and P0.1 for I2C mode
	GP0CON = BIT0 + BIT4; 			// Select I2C alternative function for P0.0 & P0.1
   
	// Enable I2C Slave mode, Slave ID and interrupt sources
	I2C0ID0  = 0xA0;				// Configure Slave's I2C0 network address
	I2C0FSTA = BIT8;				// Flush Slave Tx FIFO
	I2C0FSTA &= ~BIT8;
	I2C0SCTL = BIT0 + BIT4   		// Enable I2C Slave + Enable Rx interrupt +
			+ BIT9 + BIT10;  		// Enable Rx interrupt + Enable Tx interrupt +
	                                // Enable "Stop-after-Start-Detected" IRQ.
	IRQEN = BIT16; 					// Enable I2C0 Slave interrupts

	ucStoreCount = 0;
	while (1)
	{
	}
}

void IRQ_Handler(void) __irq 
{
	unsigned long IRQSTATUS = 0;
	unsigned int I2C0SSTATUS = 0;

	IRQSTATUS = IRQSTA;	   								// Read off IRQSTA register

	if ((IRQSTATUS & BIT16) == BIT16)					//If I2C0 Slave interrupt source
	{
	   I2C0SSTATUS = I2C0SSTA;
	   if ((I2C0SSTATUS & BIT2) == BIT2) 				// If I2C0 Master Tx IRQ
	   {
			if (ucStoreCount < 6)						// Have max 6 bytes been sent?
				I2C0STX = szStoreData[ucStoreCount++];	// Load Tx buffer
	   }
	   if ((I2C0SSTATUS & BIT3) == BIT3) 				// If I2C Master Rx IRQ
	   {
			if (ucStoreCount < 6)						// Have max 6 bytes been received?
			{
				szStoreData[ucStoreCount] = I2C0SRX;  	// Read Rx buffer
				ucStoreCount++;
			}
	   }
	   if ((I2C0SSTATUS & BIT10) == BIT10) 				// If I2C Stop after Start condition detected IRQ
	   {
			I2C0FSTA = BIT8;							// Flush Slave Tx FIFO
			I2C0FSTA &= ~BIT8;
	   }
	   if ((I2C0SSTATUS & BIT13) == BIT13) 				// If I2C Repeated Start detected IRQ
	   {
			ucStoreCount = 0;							// Reset counter Array pointer.
			I2C0FSTA = BIT8;							// Flush Slave Tx FIFO
			I2C0FSTA &= ~BIT8;
	   }
	}
}

